<?php

class CheckYourData
{

    static protected $config = null;
    static protected $trackers_data = array();

    /**
     * Init la config de Check Your Data
     *
     * @param array $config :
     *  - position: tracking_footer | tracking_header
     *  - token : token d'accès
     *  - gUA : google UA
     *  - dcUrl : url de l'appel
     */
    static public function init($config = array())
    {
        self::$config = $config;
        self::$trackers_data = "{\"ganalytics\":{\"active\":true,\"ua\":\"" . self::$config['gUA'] . "\"},\"lengow\":{\"active\":false,\"id\":\"\"},\"netaffiliation\":{\"active\":false,\"id\":\"\"}}";
    }

    /**
     * Check if CheckYourData is initialized
     * If not, trigger a warning error
     */
    static protected function triggerErrorIfNotInit()
    {
        if (is_null(self::$config)) {
            trigger_error("CheckYourData n'est pas initialisé", E_USER_WARNING);
        }
    }

    /**
     * This tag has to be on every page but the paiement page
     * @param array $param
     */
    static public function index($param = array())
    {

        self::triggerErrorIfNotInit();

        $out = "<script type=\"text/javascript\">
                    (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
                    (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
                    m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
                    })(window,document,'script','//www.google-analytics.com/analytics.js','ga');

                    // add name to avoid conflicts with other trackings (especially not universal)
                    ga('create', '" . self::$config['gUA'] . "', 'auto', {'name':'checkyourdata_ga'});
                    ga('checkyourdata_ga.require', 'displayfeatures');
                    ga('checkyourdata_ga.require', 'ec');
                    ga('checkyourdata_ga.send', 'pageview');
                </script>";

        if (isset($param['smarty'])) {
            $param['smarty']->assign(self::$config['position'], $out);
        }
    }

    /**
     * Method called on paiement success
     * @param array $param
     * @return mixed|string
     */
    static public function paiement($param = array())
    {
        $commande = new Commande;
        $categorie = new Categorie;

        $token = self::$config['token'];

        if (empty($token)) {
            return 'no token';
        }

        // data to send
        // amounts
        list($arrCommande) = $commande->get(array('client_id' => $_SESSION['client_id'], 'commande_id' => $_SESSION['commande_id']));
        $arrCmdProduit = $commande->getProduit(array('commande_id'=>$arrCommande['commande_id']));

        $total = $arrCommande['total_ttc'];
        $shipping = $arrCommande['commande_port_ttc'];
        $tax = $arrCommande['total_ttc'] - $arrCommande['total_ht'];

        // items
        $items = array();

        foreach ($arrCmdProduit as $p) {
            list($categ) = $categorie->get(array("categorie_id" => $p['categorie_id']));
            $items [$p['produit_id'] . '_' . $p['produit_attribut_id']] = array(
                'name' => $p['produit_nom'],
                'price' => $p['prix_ttc'],
                'code' => $p['produit_ref_frs'],
                'category' => $categ['categorie_nom_nom'],
                'qty' => $p['qte'],
            );
        }

        $data = array(
            'token' => $token,
            'action' => 'initOrder',
            'data' => array(
                'total' => $total,
                'tax' => $tax,
                'shipping' => $shipping,
                'cartId' => $arrCommande['commande_id'],
                'items' => json_encode($items),
            ),
        );

        $res = self::send($data);
        return $res;
    }

    /**
     * Method called when the refund of an order is validated
     * @param array $param
     * @return mixed|string
     */
    static public function partialRefund($param = array()) {
        $token = self::$config['token'];
        if (empty($token)) {
            return 'no token';
        }

        $items = array();
        foreach ($param['products'] as $product) {
            $items [$product['produit_id'] . '_' . $product['produit_attribute_id']] = $product['retour_produit_qte'];
        }

        $data = array(
            'token' => $token,
            'action' => 'partialRefund',
            'data' => array(
                'items' => json_encode($items),
                'orderId' => $param['commande_id']
            ),
        );

        $res = self::send($data);
        return $res;
    }

    /**
     * Method called when an order status is updated
     * @param array $param
     * @return mixed
     */
    static public function updateOrderStatus($param = array())
    {
        $token = self::$config['token'];
        echo 'FFFUUUU';
        if (empty($token)) {
            echo 'no token';
        }

        $res = self::sendOrderToApp($param['commande_id'], $param['new_status_id']);
        return $res;
    }

    /**
     * Method called one time to initialize the parameters of the shop
     * @return mixed
     */
    public function sendShopParamsToApp()
    {
        $statut = new Statut;
        $paiement = new Paiement;
        $texte = new Texte;

        $langue = $_SESSION['langue_id'];

        // Get order states
        $oss = $statut->getStatusList($langue);
        $states = array();
        foreach ($oss as $os) {
            $states[$os['commande_statut_id']] = $os['commande_statut_nom_nom'];
        }

        // Get payment modules
        $token = self::$config['token'];
        $modules = array();
        $pms = $paiement->get();
        foreach ($pms as $pm) {
            $modules[$pm['paiement_id']] = $pm['paiement_nom_nom'];
        }
        $confirmUrl = "recapitulatif.php";
        $confirmTitle = $texte->getOne('ref_title');

        $data = array(
            'token' => $token,
            'action' => 'setShopParams',
            'data' => array(
                'modules' => $modules,
                'states' => $states,
                'confirm_url' => $confirmUrl,
                'confirm_title' => $confirmTitle
            ),
        );
        return $this->send($data);
    }

    /**
     * @param $orderId
     * @param null $nextState
     * @return mixed
     */
    static public function sendOrderToApp($orderId, $nextState = null)
    {
        $commande = new Commande;
        $categorie = new Categorie;
        $boutique = new Boutique;
        $devise = new Devise;
        $token = self::$config['token'];
        list($myCommande) = $commande->get(array("commande_id" => $orderId));

        if (empty($myCommande)) {
            echo 'no commande';
        }

        if ($nextState == null) {
            $nextState = $myCommande['commande_statut_id'];
        }
        // Product information
        $products = $commande->getProduit(array("commande_id" => $myCommande['commande_id']));
        $items = array();
        foreach ($products as $p) {
            list($categ) = $categorie->get(array("categorie_id" => $p['categorie_id']));
            $items [$p['produit_id'] . '_' . $p['produit_attribute_id']] = array(
                'name' => $p['produit_nom'],
                'price' => $p['prix_ttc'],
                'code' => $p['produit_ref_frs'],
                'category' => $categ['categorie_nom_nom'],
                'qty' => $p['qte'],
            );
        }

        $total = $myCommande['total_ttc'];
        $shipping = (!empty($myCommande['commande_port_ttc'])) ? $myCommande['commande_port_ttc'] : 0;
        $tax = $myCommande['total_ttc'] - $myCommande['total_ht'];

        // Currency
        list($myBoutique) = $boutique->get(array("boutique_id" => $myCommande['boutique_id']));
        list($myDevise) = $devise->get(array("devise_id" => $myBoutique['devise_id']));
        $devise = $myDevise['devise_code'];
        if ($devise == 'â‚¬' || $devise == '€') {
            $devise = 'EUR';
        }

        $data = array(
            'token' => $token,
            'action' => 'changeOrderState',
            'data' => array(
                'total' => $total,
                'tax' => $tax,
                'shipping' => $shipping,
                'cartId' => $myCommande['commande_id'],
                'items' => json_encode($items),
                'orderId' => $myCommande['commande_id'],
                'state' => $nextState,
                'paymentModuleId' => $myCommande['paiement_id'],
                'currency' => $devise
            ),
        );
        $res = self::send($data);
        return $res;
    }

    /**
     * @param $k
     * @param $v
     */
    static public function addTrackerData($k, $v)
    {
        self::$trackers_data[$k] = $v;
    }

    /**
     * Encode data in json and send with APP CYD pubkey
     * @param $data : data array
     * @return string : encoded string
     */
    static public function encodeData($data)
    {
        $retour = false;

        $pubkey = openssl_pkey_get_public('file://' . dirname(__FILE__) . '/ssl/publickey.cer');

        if ($pubkey != false) {
            $ekeys = array();
            $crypted = '';
            if (openssl_seal(json_encode($data), $crypted, $ekeys, array($pubkey))) {
                $retour = array(
                    'key' => urlencode($ekeys[0]),
                    'data' => urlencode($crypted)
                );
            }
            openssl_free_key($pubkey);
        }

        return $retour;
    }

    /**
     * @param $data
     * @return mixed
     */
    static public function send($data)
    {
        if (!empty(self::$trackers_data)) {
            $data['data']['trackers_data'] = self::$trackers_data;
        }

        $enc = self::encodeData($data);

        $dat = 'k=' . $enc['key'] . '&d=' . $enc['data'];

        if (function_exists('curl_init')) {
            // if curl active
            $c = curl_init(self::$config['dcUrl']);
            curl_setopt($c, CURLOPT_POST, true);
            curl_setopt($c, CURLOPT_POSTFIELDS, $dat);
            curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
            $res = curl_exec($c);
            curl_close($c);
        } else {
            // otherwise : file_get_contents
            $opts = array('http' =>
                array(
                    'method' => 'POST',
                    'header' => 'Content-type: application/x-www-form-urlencoded',
                    'content' => $dat
                )
            );
            $context = stream_context_create($opts);
            $res = file_get_contents(self::$config['dcUrl'], false, $context);
        }

        $ret = json_decode($res, true);
        return $ret;
    }

}